# Aufgabe 1: Von den 64 möglichen Codons sind 3 Stopcodons (TGA, TAA, TAG). 64/3= 21,33

# Aufgabe 2: die drei möglichen forward-Frames und die drei auf dem komplementären Strang. 
#Beispiel: Die Sequenz 
# 5-GACTGACCCGTAC-3
# 3-CTGACTGGGCATG-5
# Frame 1: GAC CTG ACC CGT...
# Frame 2: ACT GAC...
# Frame 3: CTG ACC...
# Und jetzt noch die 3 auf dem reversen Strang:
# Frame 4: GTA CGG...
# Frame 5: TAC GGG...
# Frame 6: ACG GGT...  


#Aufgabe 3: Diese Musterlösung enthält zusätzliche, nicht im Unterricht behandelte Erweiterungen von Python.
# Die Aufgaben lassen sich aber selbstverstänlich auch ohne diese Erweiterungen bearbeiten.

#!/usr/local/bin/python
#laden der Sequenz aus einem .fasta-file in das Objekt seq_record. Dort sind dann die Sequenz, aber auch die ID gespeichert
from Bio import SeqIO
table=11
min_protein_length=50
# ************** in der folgenden Zeile steht der Name des zu untersuchenden Sequenzfiles, der bei bedarf angepasst werden muss ****************
for seq_record in SeqIO.parse("Ecolisequence1.fasta", "fasta"):
    print(seq_record.id)
    print(repr(seq_record.seq))
    print(len(seq_record))

# definition einer Funktion zum Auffinden von ORFs in der Sequenz nach dem Tutorial von Biopyhon. 
# die funktion bekommt folgendes Übergeben: eine Sequenz, die tabelle zum Translatieren, die minimal zu beachtende Proteinlänge

def find_orfs_with_trans(seq, trans_table, min_protein_length):
    #hier werden die Variablen Initialisiert: zuerst die für die zurückgegebenen Antwort, dann die in der Funktion genutzte Sequenzlänge, die hier gleich befüllt wird.
    answer = []
    seq_len = len(seq)
    # Schleife über den forwardstrand und den reversestrand
    for strand, nuc in [(+1, seq), (-1, seq.reverse_complement())]:
        #Schleife über die Frames (frame ergibt also 0,1,2)
        for frame in range(3):
            #Translation der gesamten Sequenz im entsprechenden Frame
            trans = str(nuc[frame:].translate(trans_table))
            #Längenbestimmung des Translatierten Bereiches
            trans_len = len(trans)
            aa_start = 0
            aa_end = 0
            #Schleife, die läuft solange aa_start < Translationslänge 
            while aa_start < trans_len:
                #hier wird das aktuelle translationsende gesucht, das ist beim ersten Stopcodon beginnend vom Startpunkt aus
                aa_end = trans.find("*", aa_start)
                #falls der endpunkt vor dem startpunkt ist...
                if aa_end == -1:
                    aa_end = trans_len
                # wenn die gefundene länge größer als die minimal vorgegebene Länge ist 
                if aa_end - aa_start >= min_protein_length:
                    if strand == 1:
                        start = frame + aa_start * 3
                        end = min(seq_len, frame + aa_end * 3 + 3)
                    else:
                        start = seq_len - frame - aa_end * 3 - 3
                        end = seq_len - frame - aa_start * 3
                    answer.append((start, end, strand, trans[aa_start:aa_end]))
                aa_start = aa_end + 1
    answer.sort()
    return answer

orf_list = find_orfs_with_trans(seq_record.seq, table, min_protein_length)
print("\n Der mit +1 gekennzeichnete Strang ist der in der Testsequenz angegebene, der mit -1 gekennzeichnete ist der revers komplementäre Strang hierzu.\n Folgende ORF wurden gefunden:\n")
for start, end, strand, pro in orf_list:
    print(
        "%s...%s - length %i, strand %i, %i:%i"
        % (pro[:30], pro[-3:], len(pro), strand, start, end)
    )